/* 
   display_error - displays a formatted string in a Motif error dialog box

   Copyright (C) 2001 Tim Stadelmann

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/




/** HEADERS **/

#include <stdio.h>
#include <stdarg.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MessageB.h>

#if ENABLE_NLS
# include <libintl.h>
# define _(Text) gettext (Text)
#else
# define _(Text) Text
#endif

/** MACROS **/

#define ARGS_LENGTH 10
#define DISPLAY_ERROR_BLOCKSIZE 256


/** FUNCTIONS **/

/* Displays a formatted string in a Motif error dialog box.
   error_dialog must contain the XmMessageBox child of a DialogShell
   as returned by XmCreateErrorDialog or XmCreateWarningDialog.  The
   format and variable arguments are interpreted as by printf and
   friends.  */
Cardinal
display_error (Widget error_dialog, String format, ...)
{
  /* Declare variables for argument lists.  */
  Arg args[ARGS_LENGTH];
  Cardinal n;

  /* the argument list */
  va_list argument_list;

  /* the message displayed in the dialog box. */
  String message;

  /* the actual length of the buffer */
  int message_length;

  /* the return value of vsnprintf */
  int error;

  message = XtMalloc (DISPLAY_ERROR_BLOCKSIZE);
  message_length = DISPLAY_ERROR_BLOCKSIZE;

  /* There are two different conventions for the return value of
     snprintf and related functions.  Please refer to the
     documentation.  */
  for (;;)
    {
      va_start (argument_list, format);
      error = vsnprintf (message, message_length, format, argument_list);
      if (error == -1)
	{
	  message_length += DISPLAY_ERROR_BLOCKSIZE;
	}
      else if (error != strlen (message))
	{
	  message_length = error + 1;
	}
      else 
	{
	  break;
	}
      message = XtRealloc (message, message_length);
    }

  n = 0;
  XtSetArg (args[n], XmNmessageString,
	    XmStringCreateLocalized (message)); n++;
  XtSetValues (error_dialog, args, n);
  XtManageChild (error_dialog);

  va_end (argument_list);
}
